==> kom%2B%2B-0.2.tar.gz: kom++-0.2/tcl-clients/nilkom.tcl <==
# The main program for the nilkom client.
#
# $Id: nilkom.tcl,v 1.20 1994/07/16 19:31:25 ceder Exp $
# Copyright (C) 1994  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


set HOST kom.lysator.liu.se
set PORT 4894
if {$argc > 1} {set HOST [lindex $argv 1]}
if {$argc > 2} {set PORT [lindex $argv 2]}
if {$argc > 3} {puts "These arguments are ignored: [lrange $argv 3 end]"}

proc resource {} {
    uplevel #0 {source $kom_library/komlib.tcl}
    uplevel #0 {source $kom_library/nilkom2.tcl}
    uplevel #0 {source $kom_library/nilkom_edit.tcl}
    uplevel #0 {source $kom_library/nilkom_cmd.tcl}
    uplevel #0 {source $kom_library/nilkom_print.tcl}
}
resource

# No prefetching until after we have logged in
set prefetch_state notyet

print "Kopplar upp mot $HOST $PORT...\n"
connect $HOST $PORT
set nilkom_version {0.2}
kom_session set_client_version nilkom $nilkom_version

proc getstr {prompt} {
    nilkom_page acknowledge
    nilkom_page prompt $prompt
    set res ""
    while {[set char [nilkom_get_key]] != 13 && $char != 10} {
	switch $char {
	    4 -
	    3
	    {
		clean_exit
	    }
	    18 -
	    12
	    {
		nilkom_page prompt $prompt
		print $res
	    }
	    8 -
	    127
	    {
		set len [string length $res]

		if {$len == 0} {
		    puts -nonewline "\a"
		} else {
		    set res [string range $res 0 [expr $len - 2]]
		    nilkom_page prompt_print "\008 \008"
		}
	    }
	    default
	    {
		print [format "%c" $char]
		append res [format "%c" $char]
	    }
	}
    }
    print "\n"
    return $res
}

proc silentgetstr {prompt} {
    nilkom_page acknowledge
    nilkom_page prompt $prompt
    set res ""
    while {[set char [nilkom_get_key]] != 13 && $char != 10} {
	switch $char {
	    4 -
	    3
	    {
		clean_exit
	    }
	    18 -
	    12
	    {
		nilkom_page prompt $prompt
	    }
	    8 -
	    127
	    {
		set len [string length $res]

		if {$len == 0} {
		    puts -nonewline "\a"
		} else {
		    set res [string range $res 0 [expr $len - 2]]
		}
	    }
	    default
	    {
		append res [format "%c" $char]
	    }
	}
    }
    print "\n"
    return $res
}

proc login {} {
    global env

    set komrcname [format "%s/.komrc" $env(HOME)]
    if {[file exists $komrcname]} {
	set komfile [open $komrcname r]
	gets $komfile persno
	gets $komfile passwd

	print "Loggar in som [kom_conference name $persno]...\n"
	if {! [kom_session login $persno $passwd]} {
	    error "Inloggningen misslyckades. Kontrollera $komrcname"
	}
    } else {
	set foo 0
	while {$foo == 0} {
	    set persno [select_unique_person "Ditt namn? "]
	    if {$persno == ""} {
		error "Hejd."
	    }
	    print "[kom_conference name $persno]\n"
	    set passwd [silentgetstr "Ditt lsenord? "]
	    if {[kom_session login $persno $passwd]} {
		set foo 1
	    } else {
		puts "Fel lsen.  Frsk igen."
	    }
	}
    }
    set prefetch_state 0
}

proc select_unique_person {prompt} {
    set persno ""
    while {$persno == ""} {
	set name [getstr $prompt]
	if {$name == ""} {
	    return ""
	}
	if {[regexp {^p[ \t]+([0-9]+)$} $name dummy persno]} {
	    safe_eval {kom_person number $persno} {
		{
		    {KOM BADPERS}
		    {
			print "Person nummer $persno finns inte\n"
			set persno ""
		    }
		}
	    }
	} else {
	    set matches [filter-persons [kom_session lookup_name $name]]
	    if {[llength $matches] == 0} {
		# No match ==> offer creation of new person.
		set persno [maybe_create_new_person $name]
	    } elseif {[llength $matches] == 1} {
		# A single match ==> try to log in.
		set persno [microconf confno [lindex $matches 0]]
	    } else {
		# Many matches. Offer a menu.
		set persno [select_a_match $matches]
	    }
	}
    }
    return $persno
}

proc select_a_match {matches} {
    print "Du kan mena ngon av fljande personer:\n"
    set x 1
    foreach match $matches {
	print "$x: [kom_conference name [microconf confno $match]]\n"
	incr x
    }
    if {[more_flush "**MER** (RET, SPC, q)"]} {
	return ""
    }
    incr x -1
    set num [getstr "Vlj nummer 1-$x: "]
    if {[regexp {^[0-9]+$} $num] && $num > 0 && $num <= $x} {
	incr num -1
	return [microconf confno [lindex $matches $num]]
    } else {
	return ""
    }
}

proc maybe_create_new_person {name} {
    print "Det verkar vara frsta gngen du anvnder LysKOM; Vlkommen!\n"
    print "Kontrollera att ditt namn, $name, r rttstavat.\n"
    print "En anvndaridentitet kommer att skapas t dig.\n"
    print "Ange ett personligt lsenord.  "
    print "Tryck RETUR fr att avbryta skapandet.\n"
    set pass1 ""
    set pass2 "x"
    while {$pass1 != $pass2} {
	set pass1 [silentgetstr "Lsen? "]
	if {$pass1 == ""} {
	    return ""
	}    
	set pass2 [silentgetstr "Repetera ditt lsenord fr kontroll: "]
	if {$pass2 == ""} {
	    return ""
	}
	if {$pass1 != $pass2} {
	    print "Du skrev inte in samma lsenord.  Frsk igen, "
	    print "eller tryck RETUR fr att avbryta.\n"
	}
    }
    print "Tyvrr, nilkom klarar nnu inte av att skapa nya personer.\n"
    print "Anvnd TTY-klienten i stllet.  Vlkommen ter.\n"
    return ""
}

set quit_flag 0
set to_mark_as_read {}
set input_line {}
set review_list {}
set keyhandler commandline

login

print_prompt

==> kom%2B%2B-0.3.tar.gz: kom++-0.3/tcl-clients/nilkom.tcl <==
# The main program for the nilkom client.
#
# $Id: nilkom.tcl,v 1.31 1994/08/14 11:53:19 ceder Exp $
# Copyright (C) 1994  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

# Load all files we might use.
proc resource {} {
    uplevel #0 {source $kom_library/komlib.tcl}
    uplevel #0 {source $kom_library/nilkom2.tcl}
    uplevel #0 {source $kom_library/nilkom_edit.tcl}
    uplevel #0 {source $kom_library/nilkom_cmd.tcl}
    uplevel #0 {source $kom_library/nilkom_print.tcl}
}
resource

proc usage {} {
    global argv

    print "usage: [lindex $argv 0] \[argument\] \[host \[port\]\]\n"
    print "\n"
    print "...dr argument r ngot/ngra av fljande:\n"
    print "    --hjlp   - Skriv ut den hr hjlptexten.\n"
    print \
	"    --swascii - verstt 8-bitarstecken till lmpliga 7-bitstecken\n"
    print "    --version - Skriv ut versionsnumret.\n"
    clean_exit
}

# Parse the command line.
set got_host 0
set got_port 0
for {set i 1} {$i < $argc} {incr i} {
    set arg [lindex $argv $i]
    switch -- $arg {
	--swascii {nilkom_page translate swascii}
	--hjlp   {usage}
	# For compatibility reasons:
	--help    {usage}
	--version {
	    print "nilkom version $nilkom_version\n"
	    clean_exit
	}
	default   
	    {
		if {$got_host && $got_port} {
		    usage
		} elseif {$got_host} {
		    set PORT $arg
		    set got_port 1
		} else {
		    set HOST $arg
		    set got_host 1
		}
	    }
    }
}

# No prefetching until after we have logged in
set prefetch_state notyet

print "nilkom version $nilkom_version\n"
print "\n"
print "Kopplar upp mot $HOST $PORT...\n"
connect $HOST $PORT

kom_session set_client_version  nilkom $nilkom_version

proc getstr {prompt} {
    nilkom_page sync
    print $prompt
    set res ""
    while {[set char [nilkom_input get_key]] != 13 && $char != 10} {
	nilkom_page acknowledge
	switch $char {
	    4 -
	    3
	    {
		clean_exit
	    }
	    18 -
	    12
	    {
		print $prompt
		print $res
	    }
	    8 -
	    127
	    {
		set len [string length $res]

		if {$len == 0} {
		    nilkom_page beep
		} else {
		    nilkom_page remove [string index $res [expr $len - 1]]
		    set res [string range $res 0 [expr $len - 2]]
		}
	    }
	    default
	    {
		print [format "%c" $char]
		append res [format "%c" $char]
	    }
	}
    }
    print "\n"
    return $res
}

proc silentgetstr {prompt} {
    nilkom_page sync
    print $prompt
    set res ""
    while {[set char [nilkom_input get_key]] != 13 && $char != 10} {
	switch $char {
	    4 -
	    3
	    {
		clean_exit
	    }
	    18 -
	    12
	    {
		print $prompt
	    }
	    8 -
	    127
	    {
		set len [string length $res]

		if {$len == 0} {
		    nilkom_page beep
		} else {
		    set res [string range $res 0 [expr $len - 2]]
		}
	    }
	    default
	    {
		append res [format "%c" $char]
	    }
	}
    }
    print "\n"
    return $res
}

proc login {} {
    global env

    set komrcname [format "%s/.komrc" $env(HOME)]
    if {[file exists $komrcname]} {
	set komfile [open $komrcname r]
	gets $komfile persno
	gets $komfile passwd

	print "Loggar in som [kom_conference name $persno]...\n"
	if {! [kom_session login $persno $passwd]} {
	    error "Inloggningen misslyckades. Kontrollera $komrcname"
	}
    } else {
	set foo 0
	while {$foo == 0} {
	    set persno [select_unique_person "Ditt namn? " "" \
			    {set persno [maybe_create_new_person $name]}]
	    if {$persno == ""} {
		error "Hejd."
	    }
	    if {[llength $persno] == 1} {
		print "[kom_conference name $persno]\n"
		set passwd [silentgetstr "Ditt lsenord? "]
		if {[kom_session login $persno $passwd]} {
		    set foo 1
		} else {
		    print "Fel lsen.  Frsk igen.\n"
		}
	    } else {
		# A created person auto-logins.
		set foo 1
	    }
	}
    }
    set prefetch_state 0
}

# Return the number of a person of the users' choice.
# PROMPT is a prompt to use if more feedback from the user is needed.
# NAME is the users initial attempt, or the empty string.
# WHEN_NOTFOUND, if set, should be a TCL statement that will be
# executed when no match was found.   login uses this to offer
# creation of new persons.
proc select_unique_person {prompt name {when_notfound {}}} {
    set persno ""
    while {$persno == ""} {
	if {$name == ""} {
	    set name [getstr $prompt]
	}
	if {$name == ""} {
	    return ""
	}
	if {[regexp {^p[ \t]+([0-9]+)$} $name dummy persno]} {
	    safe_eval {kom_person number $persno} {
		{
		    {KOM BADPERS}
		    {
			print "Person nummer $persno finns inte\n"
			set persno ""
		    }
		}
	    }
	} else {
	    set matches [filter-persons [kom_session lookup_name $name]]
	    if {[llength $matches] == 0} {
		# No match.
		set persno ""
		if {$when_notfound == ""} {
		    print "Det finns ingen person som matchar $name\n"
		} else {
		    eval $when_notfound
		    set name ""
		}
	    } elseif {[llength $matches] == 1} {
		# A single match.
		set persno [microconf confno [lindex $matches 0]]
	    } else {
		# Many matches. Offer a menu.
		set persno [select_a_match $matches]
	    }
	}
    }
    return $persno
}

proc select_a_match {matches} {
    print "Du kan mena ngon av fljande:\n"
    set x 1
    foreach match $matches {
	set res [print "$x: [kom_conference name [microconf confno $match]]\n"]
	if {$res == "terminate"} {
	    return ""
	}
	incr x
    }
    incr x -1
    set num [getstr "Vlj nummer 1-$x: "]
    if {[regexp {^[0-9]+$} $num] && $num > 0 && $num <= $x} {
	incr num -1
	return [microconf confno [lindex $matches $num]]
    } else {
	return ""
    }
}

proc maybe_create_new_person {name} {
    print "Det verkar vara frsta gngen du anvnder LysKOM; Vlkommen!\n"
    print "Kontrollera att ditt namn, $name, r rttstavat.\n"
    print "En anvndaridentitet kommer att skapas t dig.\n"
    print "Ange ett personligt lsenord.  "
    print "Tryck RETUR fr att avbryta skapandet.\n"
    set pass1 ""
    set pass2 "x"
    while {$pass1 != $pass2} {
	set pass1 [silentgetstr "Lsen? "]
	if {$pass1 == ""} {
	    return ""
	}    
	set pass2 [silentgetstr "Repetera ditt lsenord fr kontroll: "]
	if {$pass2 == ""} {
	    return ""
	}
	if {$pass1 != $pass2} {
	    print "Du skrev inte in samma lsenord.  Frsk igen, "
	    print "eller tryck RETUR fr att avbryta.\n"
	}
    }
    set pno [kom_session create_person $name $pass1]
    print "$name r nu en registrerad anvndare.\n"
    print "Ditt personnummer r $pno (fast det behver du inte komma ihg)\n"
    return [list new-person $pno]
}

set quit_flag 0
set to_mark_as_read {}
set input_line {}
set review_list {}
set keyhandler commandline

login
kom_session set_doing "Kr version ${nilkom_version} av nilkom"

print_prompt
